import os
import re
import tempfile
import shutil
import subprocess


__all__ = ("get_repo", "clone")

GIT_PREFIX = ("git@", "git://", "git+")
GIT_POSTFIX = (".git",)

RE_GITHUB = re.compile(r"^gh:/?")
RE_GITLAB = re.compile(r"^gl:/?")


def get_repo(url):
    url = str(url)  # In case we have got a `pathlib.Path`
    if not (url.endswith(GIT_POSTFIX) or url.startswith(GIT_PREFIX)):
        return None

    if url.startswith("git+"):
        url = url[4:]

    url = re.sub(RE_GITHUB, "https://github.com/", url)
    url = re.sub(RE_GITLAB, "https://gitlab.com/", url)
    return url


def clone(url):
    location = tempfile.mkdtemp()
    shutil.rmtree(location)  # Path must not exists
    subprocess.check_call(["git", "clone", url, location])
    git_folder = os.path.join(location, ".git")
    shutil.rmtree(git_folder)
    return location





__all__ = (
    "prompt",
    "prompt_bool",
)


no_value = object()


def required(value):
    if not value:
        raise ValueError()
    return value


def prompt(question, default=no_value, default_show=None, validator=required, **kwargs):
    """
    Prompt for a value from the command line. A default value can be provided,
    which will be used if no text is entered by the user. The value can be
    validated, and possibly changed by supplying a validator function. Any
    extra keyword arguments to this function will be passed along to the
    validator. If the validator raises a ValueError, the error message will be
    printed and the user asked to supply another value.
    """
    if default_show:
        question += f" [{default_show}] "
    elif default and default is not no_value:
        question += f" [{default}] "
    else:
        question += " "

    while True:
        resp = input(question)
        if not resp:
            if default is None:
                return None
            if default is not no_value:
                resp = default

        try:
            return validator(resp, **kwargs)
        except ValueError as e:
            if str(e):
                print(str(e))


def prompt_bool(question, default=False, yes="y", no="n"):
    please_answer = f' Please answer "{yes}" or "{no}"'

    def validator(value):
        if value:
            value = str(value).lower()[0]
        if value == yes:
            return True
        elif value == no:
            return False
        else:
            raise ValueError(please_answer)

    if default is None:
        default = no_value
        default_show = yes + "/" + no
    elif default:
        default = yes
        default_show = yes.upper() + "/" + no
    else:
        default = no
        default_show = yes + "/" + no.upper()

    return prompt(
        question, default=default, default_show=default_show, validator=validator
    )



import os

import jinja2
from jinja2.sandbox import SandboxedEnvironment


__all__ = ("ENVOPS_DEFAULT", "JinjaRender")

ENVOPS_DEFAULT = {"autoescape": False, "keep_trailing_newline": True}


class JinjaRender(object):
    def __init__(self, src_path, data=None, filters_=None, envops=None):
        # Jinja <= 2.10 does not work with `pathlib.Path`s
        self.src_path = str(src_path)

        _envops = ENVOPS_DEFAULT.copy()
        _envops.update(envops or {})
        _envops.setdefault("loader", jinja2.FileSystemLoader(self.src_path))
        self.env = SandboxedEnvironment(**_envops)
        if data:
            self.env.globals.update(**data)
        if filters_:
            self.env.filters.update(filters_)

    def __call__(self, fullpath, **data):
        relpath = str(fullpath).replace(self.src_path, "", 1).lstrip(os.path.sep)
        tmpl = self.env.get_template(relpath)
        return tmpl.render(**data)

    def string(self, string, **data):
        tmpl = self.env.from_string(string)
        return tmpl.render(**data)



from pathlib import Path
import yaml


__all__ = ("load_config", )


def load_config(settings, user_settings=None, src_files=None):
    """
    Negotiate a group of settings from the global defaults, the user-provided
    values and config files. The keys in the `settings` argument are the only
    settings that are considered valid.

    The priority of the values is:

        user_settings > src_files > settings

    Arguments:

        settings (dict):
            All the valid settings with its default values.

        user_settings (dict):
            The settings provided by the user. These overwrite the others.
            Any key that is not in the `settings` dict is ignored.

        src_files (list of str/Path):
            List of YAML files (with extensions) from where to try to read the
            default values of the settings provided by the user.
            The first one found is the one read, the others are ignored.
            These have priority over the global defaults but are overwritten
            by the user_settings.
            Any key that is not in the `settings` dict is ignored.

    Examples:

        >>> load_config({"a": 1}, {"a": 2})
        {'a': 2}

        >>> load_config({"a": 1, "b": 2}, {"b": 3})
        {'a': 1, 'b': 3}

        >>> load_config({"a": 1}, {"b": 2})
        {'a': 1}

    """
    assert user_settings or src_files
    user_settings = user_settings or {}
    user_defaults = _load_user_defaults(src_files)
    config = {}

    for key, default in settings.items():
        value = user_settings.get(key)
        if value is not None:
            config[key] = value
            continue
        value = user_defaults.get(key)
        if value is not None:
            config[key] = value
            continue
        config[key] = default

    return config


def _load_user_defaults(src_files):
    for path in src_files or []:
        path = Path(path)
        if not path.exists():
            continue
        return yaml.safe_load(path.read_text()) or {}
    return {}
